package ink.wgink.module.activiti.service.activiti.impl;

import com.alibaba.fastjson.JSONObject;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.UpdateException;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.activiti.service.activiti.IActivitiModelService;
import ink.wgink.util.ResourceUtil;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.editor.constants.ModelDataJsonConstants;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.repository.Model;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.image.impl.DefaultProcessDiagramGenerator;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.PNGTranscoder;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: ActivitiModelServiceImpl
 * @Description: 流程模型
 * @Author: WangGeng
 * @Date: 2021/7/12 22:01
 * @Version: 1.0
 **/
@Service
public class ActivitiModelServiceImpl extends DefaultBaseService implements IActivitiModelService, ModelDataJsonConstants {

    @Autowired
    private RepositoryService repositoryService;

    @Override
    public void saveModel(String modelId, String name, String description, String json_xml, String svg_xml) throws Exception {
        Model model = repositoryService.getModel(modelId);
        if (model == null) {
            throw new SearchException("模型不存在");
        }
        if (!StringUtils.isBlank(model.getDeploymentId())) {
            throw new UpdateException("已经部署的流程无法编辑");
        }
        JSONObject modelObject = JSONObject.parseObject(model.getMetaInfo());

        modelObject.put(MODEL_NAME, name);
        modelObject.put(MODEL_DESCRIPTION, description);
        model.setMetaInfo(modelObject.toString());
        model.setName(name);
        repositoryService.saveModel(model);
        repositoryService.addModelEditorSource(model.getId(), json_xml.getBytes(ISystemConstant.CHARSET_UTF8));

        InputStream svgStream = new ByteArrayInputStream(svg_xml.getBytes(ISystemConstant.CHARSET_UTF8));
        TranscoderInput input = new TranscoderInput(svgStream);

        PNGTranscoder transcoder = new PNGTranscoder();
        // Setup output
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        TranscoderOutput output = new TranscoderOutput(outStream);

        // Do the transformation
        transcoder.transcode(input, output);
        final byte[] result = outStream.toByteArray();
        repositoryService.addModelEditorSourceExtra(model.getId(), result);
        outStream.close();

    }

    @Override
    public JSONObject getEditorJson(String modelId) throws UnsupportedEncodingException {
        Model model = repositoryService.getModel(modelId);
        if (model == null) {
            throw new SearchException("模型不存在");
        }
        JSONObject result;
        if (StringUtils.isNotEmpty(model.getMetaInfo())) {
            result = JSONObject.parseObject(model.getMetaInfo());
        } else {
            result = new JSONObject();
            result.put(MODEL_NAME, model.getName());
        }
        result.put(MODEL_ID, model.getId());
        String readTree = new String(repositoryService.getModelEditorSource(model.getId()), ISystemConstant.CHARSET_UTF8);
        JSONObject editorJson = JSONObject.parseObject(readTree);
        result.put("model", editorJson);
        return result;
    }

    @Override
    public JSONObject getStencilset() throws IOException {
        FileInputStream fileInputStream = new FileInputStream(ResourceUtil.getResourceFile("static/stencilset.json"));
        String result = IOUtils.toString(fileInputStream, ISystemConstant.CHARSET_UTF8);
        return JSONObject.parseObject(result);
    }

    @Override
    public void getProcessImage(HttpServletResponse response, String deploymentId) {
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().deploymentId(deploymentId).singleResult();
        if (processDefinition == null) {
            throw new SearchException("流程定义不存在，请检查流程部署情况");
        }
        BpmnModel bpmnModel = repositoryService.getBpmnModel(processDefinition.getId());
        DefaultProcessDiagramGenerator defaultProcessDiagramGenerator = new DefaultProcessDiagramGenerator();
        response.setContentType("image/png");
        try (InputStream inputStream = defaultProcessDiagramGenerator.generateDiagram(bpmnModel, "png", new ArrayList<>(), new ArrayList<>(), 1.0D);
             OutputStream outputStream = response.getOutputStream();
        ) {
            byte[] buf = new byte[1024];
            for (int size; (size = inputStream.read(buf)) != -1; ) {
                outputStream.write(buf, 0, size);
            }
            outputStream.flush();
        } catch (IOException e) {
            throw new SystemException("流程图生成错误", e);
        }
    }
}
